# 目录结构
```shell
├── configmap.yaml
├── gpdb.yaml
├── hengshi.yaml
├── ingress.yaml
├── metadb.yaml
├── service.yaml
└── zookeeper.yaml
```

## 部署前准备
1. 指定namespace, 比如hengshi
```shell
kubectl create namespace hengshi
```
2. 替换gpdb.yaml中的$(POD_NAMESPACE) 变量为当前namespace, 比如 hengshi
```shell
sed -i 's/$(POD_NAMESPACE)/hengshi/'
```

3. 修改pvc
* 以下服务需要修改volume:
  * 修改 storageClassName: csi-rbd, 为当前集群的storageclass
  * 修改 storage: , 为各个服务的存储大小
```shell
metadb.yaml
gpdb.yaml
zookeeper.yaml
```
## engine
### 部署(engine)
1. 初始化,并启动engine
```shell
kubectl -n hengshi apply -f gpdb.yaml
kubectl -n hengshi exec -it master-0 -- /entrypoint.sh -m initsystem
kubectl -n hengshi exec -it master-0 -- /entrypoint.sh -m startsystem
kubectl -n hengshi exec -it master-0 -- bash -c "source ~/.bashrc; /opt/hengshi/bin/engine.sh config"
```
1. 部署
```shell
kubectl -n hengshi apply -f configmap.yaml
kubectl -n hengshi apply -f service.yaml
kubectl -n hengshi apply -f zookeeper.yaml
kubectl -n hengshi apply -f metadb.yaml
kubectl -n hengshi apply -f gpdb.yaml
kubectl -n hengshi apply -f hengshi.yaml
kubectl -n hengshi apply -f ingress.yaml #视情况是否部署ingress
```

1. 安全停止服务(metadb,engine)

```shell
kubectl -n hengshi exec -it metadb-0 -- /docker-entrypoint.sh stop metadb single
kubectl -n hengshi exec -it master-0 -- /entrypoint.sh -m stopsystem
```

### 重启(engine)
```shell
kubectl -n hengshi exec -it master-0 -- /entrypoint.sh gpstop -M fast -a
kubectl -n hengshi exec -it master-0 -- /entrypoint.sh gpstart -a
```

### 扩容(engine)
1. 修改 StatefulSet/segment
```shell
kubectl -n hengshi edit StatefulSet/segment
```
* SEGMENTS字段填写扩容后所有segment 的appname (比如2个扩容到4个)
* StatefulSet/segment 的 replicas: 改到扩容后所有segment数
```
apiVersion: v1
kind: ConfigMap
metadata:
  name: greenplum
data:
  MASTER: "master-0"
  SEGMENTS: |  #4个segment的列表
    segment-0
    segment-1
    segment-2
    segment-3
...
apiVersion: apps/v1
kind: StatefulSet
metadata:
  name: segment
spec:
  replicas: 4 #例如扩容后为4个segment
```
* 然后 kubectl -n hengshi apply -f gpdb.yaml
* 然后等待所有新增和旧有segement pod的状态都变成running

1. 写 new_host_file (新增segment列表, 比如原有2个segment(0,1), 现扩容到4个segment(0,1,2,3))
```shell
kubectl -n hengshi exec -it master-0 /bin/bash
cd /opt/hsdata/ && mkdir expand && cd expand
cat <<EOF > new_host_file
segment-2
segment-3
EOF
```

1. 执行扩容操作

```shell
kubectl -n hengshi exec -it master-0 /bin/bash
cd /opt/hsdata/expand
psql postgres -c "create database expand"
gpexpand -f new_host_file  -D expand
>y
>0 #然后会生成 gpexpand_inputfile_yyyymmdd_xxxxxx 文件

gpexpand -i gpexpand_inputfile_yyyymmdd_xxxxxx -D expand
```

1. 失败回滚(engine)
```shell
kubectl -n hengshi exec -it master-0 /bin/bash
cd /opt/hsdata/expand
gpstart -aR
gpexpand -r -D expand
```

### engine数据迁移
1. 旧engine数据导出
```shell
# dump db data
kubectl exec -it $old-master-0 -- /bin/bash
source $HSDATA/engine-cluster/export-cluster.sh
pg_dumpall > /opt/hsdata/engine.back.sql
exit
```

2. copy 数据到新机器
```shell
# cp db data
kubectl cp $old-gp:/opt/hsdata/engine.back.sql engine.back.sql
kubectl cp engine.back.sql $master-0:/opt/hsdata/engine.back.sql
```

3. 导入数据到新环境
```shell
# load db data
kubectl exec -it $master-0 /bin/bash
source $HS_HOME/engine-cluster
psql postgres < /opt/hsdata/engine.back.sql
rm /opt/hsdata/engine.back.sql
```

## hengshi

## 部署hengshi服务

```shell
kubectl apply -f configmap.yaml
kubectl apply -f service.yaml
kubectl apply -f zookeeper.yaml
kubectl apply -f metadb.yaml
kubectl apply -f hengshi.yaml
```

## 3.x 搭配 gpdb5 的配置 (hengshi 2.x 升级 3.x, 不升级gpdb)

##### 修改 gpdb5 设置
* 临时编辑 master-0 配置文件
```shell
kubectl -n hengshi exec -it metadb-0 -- /bin/bash
cd /opt/hengshi/conf; test -f hengshi-sense-env.sh || cp hengshi-sense-env.sh.sample hengshi-sense-env.sh
cat<<EOF >> hengshi-sense-env.sh
GREENPLUM_QUERY_USR=hengshi_query
GREENPLUM_QUERY_PWD=query202020
GREENPLUM_ETL_USR=hengshi_etl
GREENPLUM_ETL_PWD=etl202020
QUERY_QUEUE=hengshi_query_queue
ETL_QUEUE=hengshi_etl_queue
QUERY_QUEUE_ACTIVE_NUM=10
ETL_QUEUE_ACTIVE_NUM=4
EOF
```

* 使临时配制生效
**注意**: 这个engine.sh要从 hengshi 3.x pod 中拷出来
```shell
kubectl -n hengshi get pod
kubectl cp hengshi-sense-xxxxxxxxx-xxxxx:/opt/hengshi/bin/engine.sh engine.31.sh
kubectl cp engine.31.sh master-0:/opt/hengshi/bin/engine.31.sh
kubectl -n hengshi exec -it metadb-0 -- /bin/bash
cd /opt/hengshi/bin
./engine.31.sh config
```

##### 修改hengshi配制
* 更新configmap
```
ENGINE_QUERY_USER: hengshi_query
ENGINE_QUERY_PASSWORD: query202020
ENGINE_QUERY_QUEUE: hengshi_query_queue
ENGINE_ETL_USER: hengshi_etl
ENGINE_ETL_PASSWORD: etl202020
ENGINE_ETL_QUEUE: hengshi_etl_queue
```
```shell
kubectl -n hengshi apply -f configmap.yaml
```

* 重启hengshi pod
```shell
kubectl rollout restart deployment/hengshi-sense
```

## gpdb5 升级 gpdb6

1. 停掉hengshi-sense
```
kubectl -n hengshi scale deployment/hengshi-sense --replicas=0
```
1. 确认 configmap 配置
```
HS_ENGINE_DB: postgres
HS_ENGINE_PWD: postgres
HS_ENGINE_USR: postgres
```

2. 导出 gpdb5 数据, 选定存放导出数据目录, 需要1倍当前数据大小的空闲空间, 比如: ${HSDATA}/gpbackup
```shell
export GPNS=xxxx # gpdb 的 namespace
wget http://download.hengshi.com/3rd/pivotal_greenplum_backup_restore-1.15.0-1.tar.gz
kubectl -n $GPNS cp pivotal_greenplum_backup_restore-1.15.0-1.tar.gz master-0:/opt/hengshi/lib/
kubectl -n $GPNS exec -it master-0 -- /bin/bash
export HENGSHI_HOME=/opt/hengshi
cd $HENGSHI_HOME/lib
tar -xf pivotal_greenplum_backup_restore-1.15.0-1.tar.gz -C ${HSHOME}/lib/gpdb/gpdb/ #必须执行,解压到当前的GP5软链接目录
ln ${HSDATA}/engine-cluster/gpenv ${HSDATA}/engine-cluster/export-cluster.sh
source ${HSDATA}/engine-cluster/export-cluster.sh
psql postgres -A -t -c "select 'drop view '|| viewname || ' cascade;' from pg_catalog.pg_views where schemaname NOT IN ('pg_catalog', 'information_schema', 'gp_toolkit') order by schemaname, viewname" > drop_views.sql
cat drop_views.sql
psql postgres -f drop_views.sql
psql postgres -c "drop function if exists public.safe_to_number(text)"
# backup
gpbackup --dbname postgres --backup-dir ${HSDATA}/gpbackup --compression-level 6
```


3. 停掉 gpdb5
```shell
export GPNS=xxxx # gpdb 的 namespace
kubectl -n $GPNS exec -it master-0 -- /bin/bash
source ${HSDATA}/engine-cluster/export-cluster.sh
gpstop -Mfast -a
```

4.  变更目录
以下命令在 master-0...segment-x 每个 pod 都要执行一遍
```shell
export GPNS=xxxx # gpdb 的 namespace
gppods=$(kubectl -n $GPNS get pod | awk '/(master-|segment-)[0-9]/ { print $1; }')
echo ${gppods[@]} #此处检查一下pod的列表有没有错误
for x in ${gppods[@]}; do kubectl -n $GPNS exec -it $x -- mv /opt/hsdata/engine-cluster /opt/hsdata/engine-cluster.gp5.bak; echo "$x mv /opt/hsdata/engine-cluster /opt/hsdata/engine-cluster.gp5.bak"; done
```

5. 启动 gpdb6
更新 gpdb.yaml 中 gpdb 版本，并启动。此处以 gpdb:6.2.1.1 版本为例，部署时替换成实际当时具体版本
```shell
export GPNS=xxxx # gpdb 的 namespace
sed -i 's#image: registry.hengshi.org/gpdb:.*#image: registry.hengshi.org/gpdb:6.2.1.1#' gpdb.yaml
kubectl -n $GPNS apply -f gpdb.yaml
kubectl -n $GPNS exec -it master-0 -- /entrypoint.sh -m initsystem
kubectl -n $GPNS exec -it master-0 -- /entrypoint.sh -m startsystem
kubectl -n $GPNS exec -it master-0 -- bash -c "source ~/.bashrc; /opt/hengshi/bin/engine.sh config"
```

6. 导入数据
```shell
export GPNS=xxxx # gpdb 的 namespace
kubectl -n $GPNS cp pivotal_greenplum_backup_restore-1.15.0-1.tar.gz master-0:/opt/hengshi/lib/
kubectl -n $GPNS exec -it master-0 -- /bin/bash
export HENGSHI_HOME=/opt/hengshi
cd $HENGSHI_HOME/lib
tar -xf pivotal_greenplum_backup_restore-1.15.0-1.tar.gz -C ${HENGSHI_HOME}/lib/gpdb/gpdb/ #必须执行,解压到当前的GP5软链接目录
find ${HSDATA}/gpbackup/SegDataDir-1/backups/ -maxdepth 2 | sort
source ${HSDATA}/engine-cluster/export-cluster.sh
gprestore --backup-dir ${HSDATA}/gpbackup --timestamp xxxxxxxxxxxxxx
```

7. 恢复hengshi服务
```
kubectl -n hengshi scale deployment/hengshi-sense --replicas=N
```

### 若升级失败 GP6 -> GP5 回滚
1. 停掉服务
```shell
export GPNS=xxxx # gpdb 的 namespace
kubectl -n $GPNS exec -it master-0 -- /bin/bash
source ${HSDATA}/engine-cluster/export-cluster.sh
gpstop -Mfast -a
```

2. 删除GP6数据目录,恢复GP5目录
以下命令在 master-0...segment-x 每个 pod 都要执行一遍
```shell
export GPNS=xxxx # gpdb 的 namespace
gppods=$(kubectl -n $GPNS get pod | awk '/(master-|segment-)[0-9]/ { print $1; }')
echo ${gppods[@]} #此处检查一下pod的列表有没有错误
for x in ${gppods[@]}; do
  kubectl -n $GPNS exec -it $x -- rm /opt/hsdata/engine-cluster -rf;
  echo " $x: rm /opt/hsdata/engine-cluster -rf"
  kubectl -n $GPNS exec -it $x -- mv /opt/hsdata/engine-cluster.gp5.bak /opt/hsdata/engine-cluster;
  echo " $x: mv /opt/hsdata/engine-cluster.gp5.bak /opt/hsdata/engine-cluster"
done
```

3. 恢复GP5服务
更新 gpdb.yaml 中 gpdb 版本，并启动。此处以 gpdb:5 版本为例，部署时替换成实际当时具体版本
```shell
export GPNS=xxxx # gpdb 的 namespace
sed -i 's#image: registry.hengshi.org/gpdb:.*#image: registry.hengshi.org/gpdb:5#' gpdb.yaml
kubectl -n $GPNS apply -f gpdb.yaml
```
